// C Source File
// Created 2002-02-15; 17:20:55

#define USE_TI89              // Produce .89z File
#define USE_TI92PLUS          // Produce .9xz File

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

//#include <tigcclib.h>         // Include All Header Files

#include "COOL_LANG.h"
#include "COOL_IO.h"
#include "COOL_UTILS.h"
#include "COOL_X.h"
#include "1869.h"

#define CHOICE_SYSTEM 0
#define CHOICE_NEW 1

#define NBPORTS 40

//Boat icon
//const unsigned char BOAT_ICON[] = {0x00, 0x08, 0x1C, 0x2A, 0x49, 0xFF, 0x7F, 0x3E};
const unsigned char BOAT_ICON[] = {0x08, 0x1E, 0x2C, 0x48, 0x88, 0xFF, 0x7F, 0x3E};
//const unsigned char BOAT_ICON[] = {0x08, 0x18, 0x2C, 0x4A, 0x89, 0xFF, 0x7F, 0x3E};
//const unsigned char BOAT_ICON[] = {0x08, 0x18, 0x28, 0x4C, 0x8A, 0xFF, 0x7F, 0x3E};

//Flag icon
const unsigned char FLAG_ICON[] = {0x40, 0x7C, 0x78, 0x70, 0x60, 0x40, 0x40, 0x40};
const unsigned char FLAG_ICON_MASK[] = {0x41, 0x7D, 0x7B, 0x77, 0x6F, 0x5F, 0x5F, 0x5F};

void			initialize						();
void			resetGame							();
void			initializeEnvironment	();
void			showIntroduction			();
void			showMainMenu					();
void			startNewGame					();
void			saveGame							();
void			loadGame							();
int				mainLoop							();
void			auction								();
BOOL			overPort							(TCursor*, TPort*);
void			drawFlag							(TPort*);
void 			changeRegion 					(int, TString**);

TPanel *p;
TImage *background;
TGarbageCollector *gc;
TCursor *cr;

TGameSetting* gameSettings[6];
TGameSetting* gameSet;

int player;
int region;
int nbPlayers;
TPlayer* players[4];
TPort* ports[NBPORTS];
TVector* regPorts[4];

BOOL historical;

TPortInfoDialog* infoWin;
TSystemMenu* sysWin;
TNewGameDialog* setupWin;
TMainMenu* mainWin;

TShip* SCHOONER;
TShip* BARK;
TShip* STEAMSHIP;
TShip* LARGE_STEAMSHIP;
TShip* BRIG;
TShip* FRIGATE;
TShip* CLIPPER;
TShip* STEEL_CLIPPER;

// Main Function
void _main(void)
{
	int i;
	initialize();

	showIntroduction();
	
	showMainMenu();
	
	gc->finalize(gc);
}

void initialize()
{
	int i;

	gc = TGarbageCollector0();

	historical = TRUE;

	gameSettings[0] = TGameSetting0(5, 7000);
	gameSettings[1] = TGameSetting0(10, 7000);
	gameSettings[2] = TGameSetting0(15, 7000);
	gameSettings[3] = TGameSetting0(20, 7000);
	gameSettings[4] = TGameSetting0(26, 7000);
	gameSettings[5] = TGameSetting0(5, 12000);
	
	for (i = 0; i < 6; i++)
	{
		gc->bag->addElement(gc->bag, gameSettings[i]);
	}
	
	for (i = 0; i < 4; i++)
	{
		regPorts[i] = TVector0();
		gc->bag->addElement(gc->bag, regPorts[i]);
	}
	
	ports[0] = TPort0("Liverpool", 0);
	ports[0]->x = 37;
	ports[0]->y = 0;
	ports[0]->geo = EUROPE;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[0]);
	ports[1] = TPort0("London", 1);
	ports[1]->x = 52;
	ports[1]->y = 8;
	ports[1]->geo = EUROPE;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[1]);
	ports[2] = TPort0("Le Havre", 2);
	ports[2]->x = 59;
	ports[2]->y = 23;
	ports[2]->geo = EUROPE;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[2]);
	ports[3] = TPort0("New York", 3);
	ports[3]->x = 130;
	ports[3]->y = 8;
	ports[3]->geo = AMERICA;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[3]);
	ports[4] = TPort0("Savannah", 4);
	ports[4]->x = 113;
	ports[4]->y = 17;
	ports[4]->geo = AMERICA;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[4]);
	
	ports[5] = TPort0("Hamburg", 5);
	ports[5]->x = 89;
	ports[5]->y = 7;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[5]);
	ports[6] = TPort0("Amsterdam", 6);
	ports[6]->x = 76;
	ports[6]->y = 13;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[6]);
	ports[7] = TPort0("Lisbon", 7);
	ports[7]->x = 14;
	ports[7]->y = 67;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[7]);
	ports[8] = TPort0("Barcelona", 8);
	ports[8]->x = 58;
	ports[8]->y = 64;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[8]);
	ports[9] = TPort0("Triest", 9);
	ports[9]->x = 124;
	ports[9]->y = 42;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[9]);
	ports[10] = TPort0("Odessa", 10);
	ports[10]->x = 203;
	ports[10]->y = 41;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[10]);
	ports[11] = TPort0("Tunis", 11);
	ports[11]->x = 104;
	ports[11]->y = 85;
	regPorts[EUROPE]->addElement(regPorts[EUROPE], ports[11]);
	
	for (i = 5; i < 12; i++)
	{
		ports[i]->geo = EUROPE;
	}
	
	ports[12] = TPort0("New Orleans", 12);
	ports[12]->x = 78;
	ports[12]->y = 20;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[12]);
	ports[13] = TPort0("San Francisco", 13);
	ports[13]->x = 11;
	ports[13]->y = 1;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[13]);
	ports[14] = TPort0("Los Angeles", 14);
	ports[14]->x = 10;
	ports[14]->y = 16;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[14]);
	ports[15] = TPort0("Havana", 15);
	ports[15]->x = 119;
	ports[15]->y = 37;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[15]);
	ports[16] = TPort0("Puerto Bello", 16);
	ports[16]->x = 111;
	ports[16]->y = 48;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[16]);
	ports[17] = TPort0("Callao", 17);
	ports[17]->x = 114;
	ports[17]->y = 72;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[17]);
	ports[18] = TPort0("Cayenne", 18);
	ports[18]->x = 178;
	ports[18]->y = 56;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[18]);
	ports[19] = TPort0("Arica", 19);
	ports[19]->x = 140;
	ports[19]->y = 88;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[19]);
	ports[20] = TPort0("Rio de Janeiro", 20);
	ports[20]->x = 112;
	ports[20]->y = 92;
	regPorts[AMERICA]->addElement(regPorts[AMERICA], ports[20]);
	
	for (i = i; i < 21; i++)
	{
		ports[i]->geo = AMERICA;
	}
	
	ports[21] = TPort0("Port Said", 21);
	ports[21]->x = 147;
	ports[21]->y = 4;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[21]);
	ports[22] = TPort0("Accra", 22);
	ports[22]->x = 47;
	ports[22]->y = 36;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[22]);
	ports[23] = TPort0("Lagos", 23);
	ports[23]->x = 72;
	ports[23]->y = 37;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[23]);
	ports[24] = TPort0("Aden", 24);
	ports[24]->x = 224;
	ports[24]->y = 7;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[24]);
	ports[25] = TPort0("Tanga", 25);
	ports[25]->x = 177;
	ports[25]->y = 51;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[25]);
	ports[26] = TPort0("Luanda", 26);
	ports[26]->x = 88;
	ports[26]->y = 49;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[26]);
	ports[27] = TPort0("Mozambique", 27);
	ports[27]->x = 166;
	ports[27]->y = 71;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[27]);
	ports[28] = TPort0("Capetown", 28);
	ports[28]->x = 112;
	ports[28]->y = 94;
	regPorts[AFRICA]->addElement(regPorts[AFRICA], ports[28]);
	
	for (i = i; i < 29; i++)
	{
		ports[i]->geo = AFRICA;
	}
	
	ports[29] = TPort0("Diu", 29);
	ports[29]->x = 2;
	ports[29]->y = 16;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[29]);
	ports[30] = TPort0("Bombay", 30);
	ports[30]->x = 17;
	ports[30]->y = 23;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[30]);
	ports[31] = TPort0("Colombo", 31);
	ports[31]->x = 32;
	ports[31]->y = 40;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[31]);
	ports[32] = TPort0("Rangoon", 32);
	ports[32]->x = 85;
	ports[32]->y = 24;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[32]);
	ports[33] = TPort0("Singapore", 33);
	ports[33]->x = 111;
	ports[33]->y = 48;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[33]);
	ports[34] = TPort0("Macao", 34);
	ports[34]->x = 134;
	ports[34]->y = 18;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[34]);
	ports[35] = TPort0("Shanghai", 35);
	ports[35]->x = 142;
	ports[35]->y = 7;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[35]);
	ports[36] = TPort0("Mandao", 36);
	ports[36]->x = 165;
	ports[36]->y = 49;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[36]);
	ports[37] = TPort0("Perth", 37);
	ports[37]->x = 148;
	ports[37]->y = 88;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[37]);
	ports[38] = TPort0("Melbourne", 38);
	ports[38]->x = 197;
	ports[38]->y = 93;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[38]);
	ports[39] = TPort0("Sidney", 39);
	ports[39]->x = 213;
	ports[39]->y = 88;
	regPorts[ASIA]->addElement(regPorts[ASIA], ports[39]);
	
	for (i = i; i < 40; i++)
	{
		ports[i]->geo = ASIA;
	}
	
	//////////////////////////////
	//Parce que ports de dpart
	//////////////////////////////
	ports[0]->politic = POLITIC_STABLE;
	ports[0]->country = s("England");
	ports[1]->politic = POLITIC_STABLE;
	ports[1]->country = s("England");
	ports[2]->politic = POLITIC_STABLE;
	ports[2]->country = s("France");
	ports[3]->politic = POLITIC_STABLE;
	ports[3]->country = s("United States");
	ports[4]->politic = POLITIC_STABLE;
	ports[4]->country = s("United States");
	//////////////////////////////	
	
	for (i = 0; i < NBPORTS; i++)
	{
		gc->bag->addElement(gc->bag, ports[i]);
	}
	
	SCHOONER 				= TShip0(300, Ship_HULL_WOOD, 0, 12, 78, 4000, Ship_TYPE_SCHOONER);
	gc->bag->addElement(gc->bag, SCHOONER);
	BARK 						= TShip0(450, Ship_HULL_WOOD, 0, 14, 88, 6000, Ship_TYPE_BARK);
	gc->bag->addElement(gc->bag, BARK);
	STEAMSHIP 			= TShip0(399, Ship_HULL_WOOD, 0, 12, 85, 8000, Ship_TYPE_STEAMSHIP);
	gc->bag->addElement(gc->bag, STEAMSHIP);
	LARGE_STEAMSHIP = TShip0(699, Ship_HULL_IRON, 30, 19, 104, 14000, Ship_TYPE_LARGE_STEAMSHIP);
	gc->bag->addElement(gc->bag, LARGE_STEAMSHIP);
	BRIG 						= TShip0(600, Ship_HULL_IRON, 0, 19, 97, 8000, Ship_TYPE_BRIG);
	gc->bag->addElement(gc->bag, BRIG);
	FRIGATE 				= TShip0(999, Ship_HULL_IRON, 40, 25, 123, 16000, Ship_TYPE_FRIGATE);
	gc->bag->addElement(gc->bag, FRIGATE);
	CLIPPER					= TShip0(1197, Ship_HULL_WOOD, 45, 27, 135, 19000, Ship_TYPE_CLIPPER);
	gc->bag->addElement(gc->bag, CLIPPER);
	STEEL_CLIPPER	 	= TShip0(1500, Ship_HULL_IRON, 0, 31, 154, 21000, Ship_TYPE_STEEL_CLIPPER);
	gc->bag->addElement(gc->bag, STEEL_CLIPPER);
	
	initializeEnvironment();
}

void initializeEnvironment()
{
	FontSetSys (F_6x8);
	
	p = TPanel0();
	p->setColor(p, Panel_COLOR_BLACK);
	p->setBounds(p, 0, 0, LCD_WIDTH - 1, LCD_HEIGHT - 1);
	p->setVisible(p, TRUE);
	gc->bag->addElement(gc->bag, p);
	
	background = TImage0();
	p->add(p, background);
	gc->bag->addElement(gc->bag, background);
	
	cr = TCursor0();
	cr->move(cr, 50, 50);
	gc->bag->addElement(gc->bag, cr);
}

void showIntroduction ()
{
	TString *image;

	image = s("intro1");
	background->loadVAR(background, image, 230, 102);
	p->center(p, background);
	background->setVisible(background, TRUE);
	image->finalize(image);
	
	OSFreeTimer(1);
	OSRegisterTimer(1, 60);
	while (!OSTimerExpired(1) && !kbhit());
	
	image = s("intro2");
	background->loadVAR(background, image, 230, 102);
	image->finalize(image);

	OSFreeTimer(1);
	OSRegisterTimer(1, 60);
	while (!OSTimerExpired(1) && !kbhit());

	image = s("intro3");
	background->loadVAR(background, image, 230, 102);
	image->finalize(image);
	
	OSFreeTimer(1);
	OSRegisterTimer(1, 80);
	while (!OSTimerExpired(1))
	{
		if (kbhit())
		{
			ngetchx();
		 	break;
		}
	}
	
	background->setVisible(background, FALSE);
}

void showMainMenu ()
{
	int choice 			 = SystemMenu_CHOICE_NONE;
	TString *str1		 = s("Press the spacebar or \"y\"  to load");
	TString *str2		 = s("an old game.");
	TString *strFin	 = s("Finalizing ...");
	TString *image 	 = s("desk");
	TGarbageCollector *lgc = TGarbageCollector0();
	
	background->loadVAR(background, image, 230, 102);
	background->setVisible(background, TRUE);

	lgc->bag->addElement(lgc->bag, image);
	lgc->bag->addElement(lgc->bag, str1);
	lgc->bag->addElement(lgc->bag, str2);
	lgc->bag->addElement(lgc->bag, strFin);

	mainWin = TMainMenu0();
	p->center(p, mainWin);
	mainWin->setVisible(mainWin, TRUE);
	mainWin->line1->setVisible(mainWin->line1, TRUE);
	mainWin->line2->setVisible(mainWin->line2, TRUE);
	mainWin->line1->setCaption(mainWin->line1, mainWin->strLoading);
	gc->bag->addElement(gc->bag, infoWin);
	
	infoWin = TPortInfoDialog0(ports);
	p->center(p, infoWin);
	gc->bag->addElement(gc->bag, infoWin);

	sysWin = TSystemMenu0(&historical, cr);
	p->center(p, sysWin);
	gc->bag->addElement(gc->bag, sysWin);
	
	setupWin = TNewGameDialog0(cr, ports);
	p->center(p, setupWin);
	setupWin->move(setupWin, setupWin->x, 0);
	gc->bag->addElement(gc->bag, setupWin);
	mainWin->line1->setCaption(mainWin->line1, str1);
	mainWin->line2->setCaption(mainWin->line2, str2);

	do
	{
		randomize();
		
		if (kbhit())
		{
			choice = ngetchx();
			
			if (choice == 'y' || choice == 'Y')
			{
				choice = CHOICE_SYSTEM;
			}
			else /*if (choice == ' ')*/
			{
				choice = CHOICE_NEW;
			}
			/*else
			{
				choice = CHOICE_NONE;
			}*/
		}
		
		switch (choice)
		{
			case CHOICE_SYSTEM:
			{
				choice = sysWin->show(sysWin, SystemMenu_OPTION_NO_SAVE);
				
				break;
			}
			case CHOICE_NEW:
			{
				mainWin->setVisible(mainWin, FALSE);
				startNewGame();
				if (nbPlayers > 1)
				{
					auction();
				}
				break;
			}
		}
		
		if (choice == SystemMenu_CHOICE_LOAD)
		{
			mainWin->line2->setCaption(mainWin->line2, mainWin->strNull);
			mainWin->line1->setCaption(mainWin->line1, mainWin->strLoad);
			loadGame();
			mainWin->setVisible(mainWin, FALSE);
		}
		
		if (choice == SystemMenu_CHOICE_LOAD || choice == CHOICE_NEW)
		{
			background->setVisible(background, FALSE);
			
			choice = mainLoop();
			
			if (choice != SystemMenu_CHOICE_QUIT)
			{
				background->loadVAR(background, image, 230, 102);
				p->center(p, background);
				background->setVisible(background, TRUE);
				mainWin->line1->setCaption(mainWin->line1, str1);
				mainWin->line2->setCaption(mainWin->line2, str2);
				mainWin->setVisible(mainWin, TRUE);
			}
		}
	}
	while (choice != SystemMenu_CHOICE_QUIT);

	lgc->finalize(lgc);
}

void startNewGame ()
{
	TString *strPlayer		 = s(" - Player %0");
	TString *strTitle;
	TString *strNo;
	TGarbageCollector *lgc = TGarbageCollector0();
	BOOL exit = FALSE;
	int i;
	TBranch *branch;
	
	lgc->bag->addElement(lgc->bag, strPlayer);
	
	setupWin->setVisible(setupWin, TRUE);
	setupWin->lblPlayers->setVisible(setupWin->lblPlayers, TRUE);
	setupWin->lblLong->setVisible(setupWin->lblLong, TRUE);
	setupWin->btnOk->setVisible(setupWin->btnOk, TRUE);
	setupWin->cbPlayers->setVisible(setupWin->cbPlayers, TRUE);
	setupWin->cbLong->setVisible(setupWin->cbLong, TRUE);
	
	while (!exit)
	{
		cr->use(cr);
		if (cr->isOver(cr, setupWin->cbPlayers))
		{
			setupWin->cbPlayers->use(setupWin->cbPlayers);
		}
		else if (cr->isOver(cr, setupWin->cbLong))
		{
			setupWin->cbLong->use(setupWin->cbLong);
		}
		else if (cr->isOver(cr, setupWin->btnOk))
		{
			exit = TRUE;
		}
	}
	
	nbPlayers  = setupWin->cbPlayers->items->indexOf1(setupWin->cbPlayers->items, setupWin->cbPlayers->selected) + 1;
	gameSet		 = gameSettings[setupWin->cbLong->items->indexOf1(setupWin->cbLong->items, setupWin->cbLong->selected)];

	setupWin->lblPlayers->setVisible(setupWin->lblPlayers, FALSE);
	setupWin->lblLong->setVisible(setupWin->lblLong, FALSE);
	setupWin->cbPlayers->setVisible(setupWin->cbPlayers, FALSE);
	setupWin->cbLong->setVisible(setupWin->cbLong, FALSE);
	
	for (i = 0; i < nbPlayers; i++)
	{
		setupWin->cbPort->setSelected(setupWin->cbPort, ports[0]);
		setupWin->cbSex->setSelected(setupWin->cbSex, setupWin->str41);
		strNo = Integer_toString(i + 1);
		strTitle = makeString(strPlayer, 1, strNo);
		setupWin->lblNoPlayer->setCaption(setupWin->lblNoPlayer, strTitle);
		strNo->finalize(strNo);
		strTitle->finalize(strTitle);
		
		setupWin->txtName->setCaption(setupWin->txtName, setupWin->str0);
		setupWin->txtComp->setCaption(setupWin->txtComp, setupWin->str0);
		
		exit = FALSE;
		
		setupWin->lblName->setVisible(setupWin->lblName, TRUE);
		setupWin->lblPort->setVisible(setupWin->lblPort, TRUE);
		setupWin->lblComp->setVisible(setupWin->lblComp, TRUE);
		setupWin->lblNoPlayer->setVisible(setupWin->lblNoPlayer, TRUE);
		setupWin->cbPort->setVisible(setupWin->cbPort, TRUE);
		setupWin->txtName->setVisible(setupWin->txtName, TRUE);
		setupWin->cbSex->setVisible(setupWin->cbSex, TRUE);
		setupWin->txtComp->setVisible(setupWin->txtComp, TRUE);
		
		while (!exit)
		{
			cr->use(cr);
			if (cr->isOver(cr, setupWin->cbPort))
			{
				setupWin->cbPort->use(setupWin->cbPort);
			}
			else if (cr->isOver(cr, setupWin->txtName))
			{
				setupWin->txtName->use(setupWin->txtName);
			}
			else if (cr->isOver(cr, setupWin->cbSex))
			{
				setupWin->cbSex->use(setupWin->cbSex);
			}
			else if (cr->isOver(cr, setupWin->txtComp))
			{
				setupWin->txtComp->use(setupWin->txtComp);
			}			
			else if (cr->isOver(cr, setupWin->btnOk))
			{
				if ((setupWin->txtName->caption->length(setupWin->txtName->caption) > 0) && (setupWin->txtComp->caption->length(setupWin->txtComp->caption) > 0))
				{
					exit = TRUE;
					players[i] = TPlayer0(setupWin->txtName->caption, setupWin->txtComp->caption, ((setupWin->cbSex->items->indexOf1(setupWin->cbSex->items, setupWin->cbSex->selected) == 0) ? 'm' : 'f'));
					players[i]->startPort = setupWin->cbPort->selected;
					branch = TBranch0(players[i]->startPort);
					branch->money = gameSet->startingMoney;
					players[i]->branches->addElement(players[i]->branches, branch);
				}
			}
		}
	}
	
	setupWin->lblNoPlayer->setVisible(setupWin->lblNoPlayer, FALSE);
	setupWin->lblPort->setVisible(setupWin->lblPort, FALSE);
	setupWin->cbPort->setVisible(setupWin->cbPort, FALSE);
	setupWin->lblName->setVisible(setupWin->lblName, FALSE);
	setupWin->txtName->setVisible(setupWin->txtName, FALSE);
	setupWin->cbSex->setVisible(setupWin->cbSex, FALSE);
	setupWin->lblComp->setVisible(setupWin->lblComp, FALSE);
	setupWin->txtComp->setVisible(setupWin->txtComp, FALSE);
	setupWin->btnOk->setVisible(setupWin->btnOk, FALSE);
	
	FontSetSys(F_4x6);
	
	setupWin->txtQuote->setVisible(setupWin->txtQuote, TRUE);
	
	FontSetSys(F_6x8);
	
	ngetchx();
	
	setupWin->txtQuote->setVisible(setupWin->txtQuote, FALSE);
	
	setupWin->setVisible(setupWin, FALSE);
	
	mainWin->line1->setCaption(mainWin->line1, mainWin->strLoading);
	mainWin->line2->setCaption(mainWin->line2, mainWin->strNull);
	mainWin->setVisible(mainWin, TRUE);
	
	resetGame();
	
	mainWin->setVisible(mainWin, FALSE);
	
	background->setVisible(background, FALSE);
	
	lgc->finalize(lgc);
}

void saveGame ()
{
	int i;
	TFile* file = TFile1(sysWin->txtFile->caption);
	
	file->open(file);
	
	file->writeInt(file, 12345);
	
	file->writeInt(file, player);
	
	for (i = 0; i < NBPORTS; i++)
	{
		file->writeInt(file, ports[i]->politic);
		file->writeString(file, ports[i]->country);
	}

	file->finalize(file);
}

void loadGame ()
{
	int i;
	TFile* file = TFile1(sysWin->txtFile->caption);
	
	file->mode = File_READ;
	
	file->open(file);
	
	file->readInt(file);
	
	player = file->readInt(file);
	
	for (i = 0; i < NBPORTS; i++)
	{
		if (ports[i]->country != NULL)
		{
			ports[i]->country->finalize(ports[i]->country);
		}
	}
	
	for (i = 0; i < NBPORTS; i++)
	{
		ports[i]->politic = file->readInt(file);
		ports[i]->country = file->readString(file);
	}

	file->finalize(file);
}

int mainLoop ()
{
	TString *strSpr = s("$");
	TString *strWorld = s("world");
	TString *strBlank = s("blank");
	TString* mp[4];
	TString *mpAmerica = s("mpame");
	TString *mpEurope = s("mpeur");
	TString *mpAfrica = s("mpafr");
	TString *mpAsia = s("mpasi");
	TContainer *c	 = TContainer0();
	TImage *map = TImage2(strWorld, 49, 15);
	TComponent* regions[4];
	TLabel *lblDate = TLabel2("01|JAN|1854");
	TString* strMoney0;
	TString* strMoney1;
	TLabel *lblMoney = TLabel0();
	TLabel *lblComp = TLabel0();
	TString *strBank = s("ibank");
	TString *strRecords = s("irec");
	TString *strInfo = s("iinfo");
	TString *strMenu = s("imenu");
	TImage *btnBank = TImage2(strBank, 16, 16);
	TImage *btnRecords = TImage2(strRecords, 16, 16);
	TImage *btnInfo = TImage2(strInfo, 16, 16);
	TImage *btnMenu = TImage2(strMenu, 16, 16);
	TPort *port;
	TShip *ship;
	int i;
	int choice;
	int total;
	BOOL exit = FALSE;
	BOOL newTurn = TRUE;
	BOOL foundShip;
	TGarbageCollector *lgc = TGarbageCollector0();
	
	mainWin->line1->setCaption(mainWin->line1, mainWin->strLoading);
	mainWin->line2->setCaption(mainWin->line2, mainWin->strNull);
	mainWin->setVisible(mainWin, TRUE);
	
	lgc->bag->addElement(lgc->bag, lblDate);
	lgc->bag->addElement(lgc->bag, lblMoney);
	lgc->bag->addElement(lgc->bag, c);
	lgc->bag->addElement(lgc->bag, strWorld);
	lgc->bag->addElement(lgc->bag, strBlank);
	lgc->bag->addElement(lgc->bag, mpAmerica);
	lgc->bag->addElement(lgc->bag, mpEurope);
	lgc->bag->addElement(lgc->bag, mpAfrica);
	lgc->bag->addElement(lgc->bag, mpAsia);
	lgc->bag->addElement(lgc->bag, btnBank);
	lgc->bag->addElement(lgc->bag, btnRecords);
	lgc->bag->addElement(lgc->bag, btnInfo);
	lgc->bag->addElement(lgc->bag, btnMenu);
	lgc->bag->addElement(lgc->bag, strBank);
	lgc->bag->addElement(lgc->bag, strRecords);
	lgc->bag->addElement(lgc->bag, strInfo);
	lgc->bag->addElement(lgc->bag, strMenu);
	lgc->bag->addElement(lgc->bag, strSpr);
	lgc->bag->addElement(lgc->bag, map);
	
	c->add(c, map);
	
	for (i = 0; i < 4; i++)
	{
		regions[i] = TComponent0();
		c->add(c, regions[i]);
		lgc->bag->addElement(lgc->bag, regions[i]);
	}
	
	//America
	regions[AMERICA]->setBounds(regions[AMERICA], 0, 0, 17, 15);
	mp[AMERICA] = mpAmerica;
	//Europe
	regions[EUROPE]->setBounds(regions[EUROPE], 17, 0, 35, 5);
	mp[EUROPE] = mpEurope;
	//Africa
	regions[AFRICA]->setBounds(regions[AFRICA], 17, 5, 35, 15);
	mp[AFRICA] = mpAfrica;
	//Asia
	regions[ASIA]->setBounds(regions[ASIA], 35, 0, 49, 15);
	mp[ASIA] = mpAsia;

	background->loadVAR(background, strBlank,  8, 8);
	background->setBounds(background, 0, 0,  229, 101);
	p->center(p, background);
	background->move(background, background->x, 1);

	p->add(p, c);
	p->add(p, lblComp);
	
	c->width  = map->width;
	c->height = map->height;
	
	p->center(p, c);
	
	c->move(c, c->x, 105);
	
	lblComp->setOpaque(lblComp, TRUE);

	lblDate->move(lblDate, 5, 103);
	lblDate->setColor(lblDate, Label_COLOR_WHITE);

	lblMoney->move(lblMoney, 5, lblDate->height + 102);
	lblMoney->setColor(lblMoney, Label_COLOR_WHITE);

	btnBank->move(btnBank, 150, 105);
	
	btnRecords->move(btnRecords, btnBank->x + btnBank->width + 1, 105);

	btnInfo->move(btnInfo, btnRecords->x + btnRecords->width + 1, 105);

	btnMenu->move(btnMenu, btnInfo->x + btnInfo->width + 1, 105);

	mainWin->setVisible(mainWin, FALSE);

	background->setVisible(background, TRUE);

	c->setVisible(c, TRUE);
	map->setVisible(map, TRUE);
	
	lblDate->setVisible(lblDate, TRUE);
	lblMoney->setVisible(lblMoney, TRUE);
	
	btnBank->setVisible(btnBank, TRUE);
	btnRecords->setVisible(btnRecords, TRUE);
	btnInfo->setVisible(btnInfo, TRUE);
	btnMenu->setVisible(btnMenu, TRUE);
	
	while (!exit)
	{
		if (newTurn)
		{
			lblComp->setCaption(lblComp, players[player]->company);
			p->center(p, lblComp);
			lblComp->setY(lblComp, 0);
			
			total = 0;
			for (i = 0; i < players[player]->branches->size; i++)
			{
				total = total + ((TBranch*)players[player]->branches->elementAt(players[player]->branches, i))->money;
			}
			for (i = 0; i < players[player]->ships->size; i++)
			{
				total = total + ((TShip*)players[player]->ships->elementAt(players[player]->ships, i))->money;
			}
			strMoney0 = Float_toString(total);
			strMoney1 = strSpr->concat(strSpr, strMoney0);
			lblMoney->setCaption(lblMoney, strMoney1);
			strMoney0->finalize(strMoney0);
			strMoney1->finalize(strMoney1);
			
			foundShip = FALSE;
			
			for (i = 0; (i < players[player]->ships->size) && !foundShip; i++)
			{
				ship = (TShip*)players[player]->ships->elementAt(players[player]->ships, i);
				if (ship->port != NULL)
				{
					foundShip = TRUE;
				}
			}
			
			if (foundShip)
			{
				port = ship->port;
			}
			else
			{
				port = ((TBranch*)players[player]->branches->elementAt(players[player]->branches, 0))->port;
			}
			
			lblComp->setVisible(lblComp, FALSE);
			changeRegion(port->geo, mp);
			lblComp->setVisible(lblComp, TRUE);
			newTurn = FALSE;
		}		
		cr->use(cr);
		if (cr->isOver(cr, map))
		{
			for(i = 0; i < 4; i++)
			{
				if (cr->isOver(cr, regions[i]) && region != i)
				{
					lblComp->setVisible(lblComp, FALSE);
					changeRegion(i, mp);
					lblComp->setVisible(lblComp, TRUE);
					break;
				}
			}
		}
		else if (cr->isOver(cr, lblDate))
		{
			newTurn = TRUE;
			if (player < nbPlayers)
			{
				player = player + 1;
			}
			else 
			{
				player = 0;
			}
		}
		else if (cr->isOver(cr, btnMenu))
		{
			choice = sysWin->show(sysWin, SystemMenu_OPTION_NORMAL);
			
			if (choice == SystemMenu_CHOICE_SAVE)
			{
				mainWin->line2->setCaption(mainWin->line2, mainWin->strNull);
				mainWin->line1->setCaption(mainWin->line1, mainWin->strSave);
				mainWin->setVisible(mainWin, TRUE);
				saveGame();
				mainWin->setVisible(mainWin, FALSE);
				choice = SystemMenu_CHOICE_CONTINUE;
			}
			else if (choice != SystemMenu_CHOICE_CONTINUE)
			{
				exit = TRUE;
			}
		}
		else
		{
			for (i = 0; i < regPorts[region]->size; i++)
			{
				port = (TPort*)regPorts[region]->elementAt(regPorts[region], i);
				if (overPort(cr, port))
				{
					infoWin->showPort(infoWin, port->id);
					break;
				}
			}
		}
	}
	
	c->setVisible(c, FALSE);
	map->setVisible(map, FALSE);
	
	background->setVisible(background, FALSE);
	
	p->contentPane->removeElement(p->contentPane, c);
	p->contentPane->removeElement(p->contentPane, lblComp);
	
	lgc->finalize(lgc);
	
	return choice;
}

void resetGame ()
{
	int i;
	
	player = 0;	
	
	for (i = 0; i < NBPORTS; i++)
	{
		if (ports[i]->country != NULL)
		{
			ports[i]->country->finalize(ports[i]->country);
		}
	}
	
	ports[0]->politic = POLITIC_STABLE;
	ports[0]->country = s("England");	
	ports[1]->politic = POLITIC_STABLE;
	ports[1]->country = s("England");
	ports[2]->politic = POLITIC_STABLE;
	ports[2]->country = s("France");
	ports[3]->politic = POLITIC_STABLE;
	ports[3]->country = s("United States");
	ports[4]->politic = POLITIC_STABLE;
	ports[4]->country = s("United States");

	ports[5]->politic = POLITIC_STABLE;
	ports[5]->country = s("Hannover");
	ports[6]->politic = POLITIC_STABLE;
	ports[6]->country = s("Netherlands");
	ports[7]->politic = POLITIC_STABLE;
	ports[7]->country = s("Portugal");
	ports[8]->politic = POLITIC_STABLE;
	ports[8]->country = s("Spain");
	ports[9]->politic = POLITIC_STABLE;
	ports[9]->country = s("Austria");
	ports[10]->politic = POLITIC_WAR;
	ports[10]->country = s("Russia");
	ports[11]->politic = POLITIC_UNREST;
	ports[11]->country = s("Tunisia");
	
	ports[12]->politic = POLITIC_STABLE;
	ports[12]->country = s("United States");
	ports[13]->politic = POLITIC_STABLE;
	ports[13]->country = s("United States");
	ports[14]->politic = POLITIC_STABLE;
	ports[14]->country = s("United States");
	ports[15]->politic = POLITIC_STABLE;
	ports[15]->country = s("Spain colony");
	ports[16]->politic = POLITIC_UNREST;
	ports[16]->country = s("Panama");
	ports[17]->politic = POLITIC_UNREST;
	ports[17]->country = s("Peru");
	ports[18]->politic = POLITIC_UNREST;
	ports[18]->country = s("French colony");
	ports[19]->politic = POLITIC_STABLE;
	ports[19]->country = s("Chile");
	ports[20]->politic = POLITIC_UNREST;
	ports[20]->country = s("Brasil");
	
	ports[21]->politic = POLITIC_STABLE;
	ports[21]->country = s("Egypt");
	ports[22]->politic = POLITIC_UNREST;
	ports[22]->country = s("English colony");
	ports[23]->politic = POLITIC_UNREST;
	ports[23]->country = s("English colony");
	ports[24]->politic = POLITIC_STABLE;
	ports[24]->country = s("English colony");
	ports[25]->politic = POLITIC_UNREST;
	ports[25]->country = s("German colony");
	ports[26]->politic = POLITIC_STABLE;
	ports[26]->country = s("Portugese colony");
	ports[27]->politic = POLITIC_UNREST;
	ports[27]->country = s("Portugese colony");
	ports[28]->politic = POLITIC_UNREST;
	ports[28]->country = s("English colony");
	
	ports[29]->politic = POLITIC_UNREST;
	ports[29]->country = s("English colony");
	ports[30]->politic = POLITIC_STABLE;
	ports[30]->country = s("English colony");
	ports[31]->politic = POLITIC_STABLE;
	ports[31]->country = s("English colony");
	ports[32]->politic = POLITIC_STABLE;
	ports[32]->country = s("English colony");
	ports[33]->politic = POLITIC_STABLE;
	ports[33]->country = s("English colony");
	ports[34]->politic = POLITIC_REVOLTS;
	ports[34]->country = s("China");
	ports[35]->politic = POLITIC_REVOLTS;
	ports[35]->country = s("China");
	ports[36]->politic = POLITIC_STABLE;
	ports[36]->country = s("Dutch colony");
	ports[37]->politic = POLITIC_STABLE;
	ports[37]->country = s("English colony");
	ports[38]->politic = POLITIC_STABLE;
	ports[38]->country = s("English colony");
	ports[39]->politic = POLITIC_STABLE;
	ports[39]->country = s("English colony");
}

void auction ()
{
	BOOL bidded;
	TString *image = s("auc");
	TString *strSp = s(" ");
	TTextArea *txtAuc = TTextArea0();
	TString *strBid;
	TString *strAuc = s("%0: %1 $%2");
	TString *strNo;
	TString *strPlayer;
	TString* str[19];
	TString* strName[4];
	TBranch *branch;
	TShip *ship;
	TGarbageCollector *lgc = TGarbageCollector0();
	float bids[4];
	float playersMoney[4];
	TLabel* bidLabels[4];
	TLabel* lblTitle = TLabel2("Max & Son Worldwide Ship Auction");
	float bid;
	float incr = 200;
	BOOL concluded;
	int i;
	int n;
	int key;
	int bidder;
	int nameCpt = 0;
	
	mainWin->line2->setCaption(mainWin->line2, mainWin->strNull);
	mainWin->line1->setCaption(mainWin->line1, mainWin->strLoading);
	mainWin->setVisible(mainWin, TRUE);
	
	str[0] = s("And here`s a 2 year old ship with 297  tons");
	str[1] = s("The 297 tonner opening bid is $%0"); //The $$
	str[2] = s("I`m waiting for you bids for the 297   tonner"); //for $$
	str[3] = s("A 297 tonner at $%0 is a gift.");
	str[4] = s("$%0 is a small price to ask.");
	str[5] = s("Such an opportunity won`t come your wayagain.");
	str[6] = s("Does anybody want this wonderful ship.");
	str[7] = s("My last offer is $%0 for the 297     tonner.");
	str[8] = s("So you didn`t want it any other way...");
	str[9] = s("So we`ll move onto the next model.");
	str[10] = s("The offers now lies at $%0.");
	str[11] = s("Is someone going to bid more?");
	str[12] = s("As said the offer is $%0.");
	str[13] = s("That`s $%0 for the first time.");
	str[14] = s("Is no one biding more than $%0?");
	str[15] = s("... for the second time.");
	str[16] = s("The 297 tonner is worth a little more  than $%0.");
	str[17] = s("....and, for the third time!!");
	str[18] = s("I congratulate on your jewel.");
	
	strName[0] = s("Blue Nose");
	strName[1] = s("Mermerus");
	strName[2] = s("Star of India");
	strName[3] = s("Cosmos");
	
	for (i = 0; i < nbPlayers; i++)
	{
		playersMoney[i] = gameSet->startingMoney;
	}
	
	for (i = 0; i < 19; i++)
	{
			lgc->bag->addElement(lgc->bag, str[i]);
	}
	
	for (i = 0; i < 4; i++)
	{
			lgc->bag->addElement(lgc->bag, strName[i]);
	}
	
	for (i = 0; i < 4; i++)
	{
		bidLabels[i] = TLabel1(strSp);
		if (i <= 1)
		{
			bidLabels[i]->move(bidLabels[i], 0, i * bidLabels[0]->height);
		}
		else
		{
			bidLabels[i]->move(bidLabels[i], 120, (i - 2) * bidLabels[0]->height);
		}
		bidLabels[i]->setColor(bidLabels[i], Label_COLOR_WHITE);
		lgc->bag->addElement(lgc->bag, bidLabels[i]);
	}
	
	txtAuc->setBounds(txtAuc, 0, 100, 239, 120);
	
	p->center(p, lblTitle);
	lblTitle->setY(lblTitle, 5);
	lblTitle->setColor(lblTitle, Label_COLOR_WHITE);
	
	p->add(p, txtAuc);
	p->add(p, lblTitle);
		
	lgc->bag->addElement(lgc->bag, strSp);
	lgc->bag->addElement(lgc->bag, txtAuc);
	lgc->bag->addElement(lgc->bag, strAuc);
	lgc->bag->addElement(lgc->bag, lblTitle);
	lgc->bag->addElement(lgc->bag, image);
	
	background->loadVAR(background, image,  230, 102);
	p->center(p, background);
	background->move(background, background->x, 20);

	mainWin->setVisible(mainWin, FALSE);
	
	background->setVisible(background, TRUE);
	
	lblTitle->setVisible(lblTitle, TRUE);
	ngetchx();
	lblTitle->setVisible(lblTitle, FALSE);
	
	for (i = 0; i < nbPlayers; i++)
	{
		for (n = 0; n < nbPlayers; n++)
		{
			strNo  = Integer_toString(n + 1);
			strBid = Float_toString(playersMoney[n]);
			strPlayer = makeString(strAuc, 3, strNo, players[n]->name, strBid);
			
			bidLabels[n]->setCaption(bidLabels[n], strPlayer);
			bidLabels[n]->setVisible(bidLabels[n], TRUE);
			strPlayer->finalize(strPlayer);
			strNo->finalize(strNo);
			strBid->finalize(strBid);
		}
		
		bid = 1750;
		concluded = FALSE;
		bidded = FALSE;
		for (n = 0; n <= 9 && !bidded; n++)
		{
			strBid  = Float_toString(bid);
			strPlayer = makeString(str[n], 1, strBid);
			
			txtAuc->setCaption(txtAuc, strPlayer);
			
			strBid->finalize(strBid);
			strPlayer->finalize(strPlayer);
			if (!txtAuc->visible)
			{
				txtAuc->setVisible(txtAuc, TRUE);
			}
			OSFreeTimer(1);
			OSRegisterTimer(1, 80);
			while (!OSTimerExpired(1))
			{
				if (kbhit())
				{
					key = ngetchx();
					
					if (key >= '1' && key <= nbPlayers + '0')
					{
						if (playersMoney[key - '1'] >= bid)
						{
							bid = bid + incr;
							bids[key - '1'] = bid;
							bidded = TRUE;
							bidder = key - '1';
							break;
						}
					}
				}
			}
		}
		
		if (bidded)
		{
			while (!concluded)
			{
				bidded = FALSE;
				for (n = 10; n <= 17 && !bidded; n++)
				{
					strBid  = Float_toString(bid);
					strPlayer = makeString(str[n], 1, strBid);
					
					txtAuc->setCaption(txtAuc, strPlayer);
					
					strBid->finalize(strBid);
					strPlayer->finalize(strPlayer);
					
					OSFreeTimer(1);
					OSRegisterTimer(1, 80);
					while (!OSTimerExpired(1))
					{
						if (kbhit())
						{
							key = ngetchx();
							if (key >= '1' && key <= nbPlayers + '0')
							{
								if (playersMoney[key - '1'] >= bid + incr)
								{
									bid = bid + incr;
									bids[key - '1'] = bid;
									bidded = TRUE;
									bidder = key - '1';
								}
							}
						}
					}
					
					if (n == 17)
					{
						concluded = TRUE;
					}
				}
			}

			txtAuc->setCaption(txtAuc, str[n]);
			OSFreeTimer(1);
			OSRegisterTimer(1, 40);
			while (!OSTimerExpired(1));
			playersMoney[bidder] = playersMoney[bidder] - bid;
		
			branch = (TBranch*)players[bidder]->branches->elementAt(players[bidder]->branches, 0);
		
			branch->money = playersMoney[bidder];
			
			ship = SCHOONER->clone(SCHOONER);
			
			ship->name = c(strName[nameCpt++]);

			ship->port = branch->port;
			
			players[bidder]->ships->addElement(players[bidder]->ships, ship);
		}
	}
	
	p->contentPane->removeElement(p->contentPane, txtAuc);
	p->contentPane->removeElement(p->contentPane, lblTitle);

	txtAuc->setVisible(txtAuc, FALSE);

	for (i = 0; i < 4; i++)
	{
		bidLabels[i]->setVisible(bidLabels[i], FALSE);
	}

	background->setVisible(background, FALSE);
	
	lgc->finalize(lgc);
}

BOOL overPort (TCursor* cr, TPort* port)
{
	BOOL result;
	
	if (cr->x >= port->x + background->x &&
		  cr->y >= port->y + background->y &&
		  cr->x < port->x + background->x + 8 &&
		  cr->y < port->y + background->y + 8)
	{
		result = TRUE;
	}
	else
	{
		result = FALSE;
	}
	
	return result;
}

void drawFlag (TPort *port)
{
	Sprite8(port->x + background->x + 8, port->y + background->y, 8, FLAG_ICON_MASK, LCD_MEM, SPRT_AND);
	Sprite8(port->x + background->x + 8, port->y + background->y, 8, FLAG_ICON, LCD_MEM, SPRT_OR);
}

void changeRegion (int i, TString** mp)
{
	int n;
	int z;
	TBranch *branch;
	TPort *port;
	region = i;
	background->loadVAR(background, mp[i],  230, 102);
	for (n = 0; n < players[player]->branches->size; n++)
	{
		branch = (TBranch*)players[player]->branches->elementAt(players[player]->branches, n);
		for (z = 0; z < regPorts[region]->size; z++)
		{
			port = (TPort*)regPorts[region]->elementAt(regPorts[region], z);
			if (branch->port == port)
			{
				drawFlag(port);
				ngetchx();
			}
		}
	}
}
