/*
QUSOFT MICROSYSTMES
1869

Classe: TSystemMenu - Type SystemMenu

Hirarchie:
TObject -> TComponent -> TContainer -> TFrame -> TSystemMenu
*/

/*
		Copyright 2001 Frdric Brown
*/

/*
    This file is part of Gundealer.

    Gundealer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gundealer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gundealer; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * TSystemMenu class represents the system nenu used to load or save a game.
*/

#define SystemMenu_OPTION_NO_SAVE 0
#define SystemMenu_OPTION_NORMAL 1

#define SystemMenu_CHOICE_NONE -1
#define SystemMenu_CHOICE_QUIT 2
#define SystemMenu_CHOICE_LOAD 3
#define SystemMenu_CHOICE_RESTART 4
#define SystemMenu_CHOICE_CONTINUE 5
#define SystemMenu_CHOICE_SAVE 6

//Structure de l'objet de Type SystemMenu  extension de TFrame
typedef struct SystemMenu  {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Frame*	(*clone)		(struct Frame*);
	BOOL					(*equals)		(struct Frame*, struct Object*);
	void					(*finalize)	(struct SystemMenu*);
	TString* 			(*toString) (struct Frame*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)		 	(struct Frame*);
	void							(*setBounds)	(struct Container*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Container*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* contentPane;	//Le contenu du Container
	BOOL* visibility;			//La visibilit des composants
	void							(*bringOn)		(struct Container*, TComponent*);
	void							(*center)			(struct Container*, TComponent*);
	void							(*drag)				(struct Container*, TComponent*, TCursor*);
	TString* title;				//Le titre de la fentre
	void							(*setTitle)		(struct Frame*, TString*);
	void							(*add)				(struct Container*, TComponent*);
	TString *str0;
	TString *strOn;
	TString *strOff;
	TString *strInvalid;
	BOOL* historical;
	TTextField *txtFile;
	TButton *btnLoad;
	TCheckButton *btnSave;
	TButton *btnRestart;
	TButton *btnExit;
	TButton *btnHisto;
	TButton *btnContinue;
	TLabel *lblFile;
	TCursor* cr;
	TGarbageCollector* gc;
	int								(*show)				(struct SystemMenu*, int);
} TSystemMenu ;

//Dclaration des constructeurs
TSystemMenu* TSystemMenu0(BOOL*, TCursor*);

//Dclaration des mthodes
int		TSystemMenu_show				(TSystemMenu*, int);
void 	TSystemMenu_finalize		(TSystemMenu*);

/**
 * Ce constructeur instancie un objet de Type SystemMenu.
 @since Release 1.0
*/
TSystemMenu* TSystemMenu0 (BOOL* historical, TCursor* cr) {
	TSystemMenu *this = (TSystemMenu*)malloc(sizeof(TSystemMenu));
	
	int i;
	
	this->class  = "TSystemMenu";
	this->super  = "TFrame";
	
	this->height 			= 105;
	this->visible 		= FALSE;
	this->width 			= 200;
	this->x						= 0;
	this->y						= 0;
	this->img					= (BITMAP*)malloc(sizeof(BITMAP));
	this->contentPane = TVector0();
	this->visibility	= NULL;
	this->title				= s("System Menu");
	
	this->clone 	 = TFrame_clone;
	this->equals 	 = TFrame_equals;
	this->finalize = TSystemMenu_finalize;
	this->toString = TFrame_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TFrame_print;
	this->setBounds  = TContainer_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TContainer_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	
	this->bringOn		 = TContainer_bringOn;
	this->center		 = TContainer_center;
	this->drag			 = TContainer_drag;
	this->add				 = TContainer_add;

	this->setTitle	 = TFrame_setTitle;

	this->show			 = TSystemMenu_show;

	this->historical = historical;
	
	this->gc				 = TGarbageCollector0();
	
	this->cr				 = cr;
	
	this->str0		= s("");
	this->gc->bag->addElement(this->gc->bag, this->str0);
	this->strOn 	= s("Historical ON ");
	this->gc->bag->addElement(this->gc->bag, this->strOn);
	this->strOff	= s("Historical OFF");
	this->gc->bag->addElement(this->gc->bag, this->strOff);
	this->strInvalid	= s("NotFound");
	this->gc->bag->addElement(this->gc->bag, this->strInvalid);	
	
	this->txtFile		 	= TTextField2("12345678");
	this->btnLoad			= TButton2("Load");
	this->btnSave	 		= TCheckButton2("Save");
	this->btnRestart	= TButton2("Restart");
	this->btnExit			= TButton2("Exit");
	this->btnHisto		= TButton1(this->strOn);
	this->btnContinue	= TButton2("Continue");
	this->lblFile			= TLabel2("File name:");
	
	//this->setBounds(this, 0, 0, 200, 105);

	this->lblFile->move(this->lblFile, 1, 25);
	this->add(this, this->lblFile);
	this->gc->bag->addElement(this->gc->bag, this->lblFile);
	
	this->txtFile->setCaption(this->txtFile, this->str0);
	this->txtFile->move(this->txtFile, this->lblFile->x + this->lblFile->width, 25);
	this->add(this, this->txtFile);
	this->gc->bag->addElement(this->gc->bag, this->txtFile);
	
	this->btnLoad->move(this->btnLoad, 2, 45 + 2);
	this->add(this, this->btnLoad);
	this->gc->bag->addElement(this->gc->bag, this->btnLoad);
	
	this->btnSave->move(this->btnSave, this->btnLoad->x + this->btnLoad->width + 2, 45 + 2);
	this->add(this, this->btnSave);
	this->gc->bag->addElement(this->gc->bag, this->btnSave);
	
	this->center(this, this->btnHisto);
	this->btnHisto->setY(this->btnHisto, 75 + 6);
	this->add(this, this->btnHisto);
	this->gc->bag->addElement(this->gc->bag, this->btnHisto);
	
	this->btnExit->move(this->btnExit, 2, 60 + 4);
	this->add(this, this->btnExit);
	this->gc->bag->addElement(this->gc->bag, this->btnExit);
	
	this->btnRestart->move(this->btnRestart, this->btnExit->x + this->btnExit->width + 2, 60 + 4);
	this->add(this, this->btnRestart);
	this->gc->bag->addElement(this->gc->bag, this->btnRestart);
	
	this->btnContinue->move(this->btnContinue, this->btnRestart->x + this->btnRestart->width + 2, 60 + 4);
	this->add(this, this->btnContinue);
	this->gc->bag->addElement(this->gc->bag, this->btnContinue);

	return this;
}

void TSystemMenu_finalize (TSystemMenu* this)
{
	this->setVisible(this, FALSE);
	
	TRY {
		this->contentPane->finalize(this->contentPane);
		this->gc->finalize(this->gc);
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		this->title->finalize(this->title);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

int TSystemMenu_show (TSystemMenu* this, int option)
{
	TFile *file;
	int choice 						 = SystemMenu_CHOICE_NONE;
	
	this->txtFile->setCaption(this->txtFile, this->str0);
	
	if (option == SystemMenu_OPTION_NO_SAVE)
	{
		this->btnSave->setChecked(this->btnSave, TRUE);
	}
	else
	{
		this->btnSave->setChecked(this->btnSave, FALSE);
	}
	
	if (*this->historical) {
		this->btnHisto->setCaption(this->btnHisto, this->strOn);
	}
	else {
		this->btnHisto->setCaption(this->btnHisto, this->strOff);
	}
	
	this->setVisible(this, TRUE);	
	this->txtFile->setVisible(this->txtFile, TRUE);
	this->btnLoad->setVisible(this->btnLoad, TRUE);
	this->btnSave->setVisible(this->btnSave, TRUE);
	this->btnRestart->setVisible(this->btnRestart, TRUE);
	this->btnExit->setVisible(this->btnExit, TRUE);
	this->btnHisto->setVisible(this->btnHisto, TRUE);
	this->btnContinue->setVisible(this->btnContinue, TRUE);
	this->lblFile->setVisible(this->lblFile, TRUE);

	while (choice == SystemMenu_CHOICE_NONE)
	{
		this->cr->use(this->cr);
		if (this->cr->isOver(this->cr, this->txtFile))
		{
			this->txtFile->use(this->txtFile);
		}
		else if (this->cr->isOver(this->cr, this->btnLoad))
		{
			if (this->txtFile->caption->length(this->txtFile->caption) > 0)
			{
				file = TFile1(this->txtFile->caption);
				
				file->mode = File_READ;
				
				file->open(file);
				
				if (file->exists(file) && (file->readInt(file) == 12345))
				{
					choice = SystemMenu_CHOICE_LOAD;
				}
				else
				{
					this->txtFile->setCaption(this->txtFile, this->strInvalid);
					ngetchx();
					this->txtFile->setCaption(this->txtFile, this->str0);
				}
				
				file->finalize(file);
			}
		}
		else if (this->cr->isOver(this->cr, this->btnSave))
		{
			if (!this->btnSave->checked && this->txtFile->caption->length(this->txtFile->caption) > 0)
			{
				choice = SystemMenu_CHOICE_SAVE;
			}
		}
		else if (this->cr->isOver(this->cr, this->btnHisto))
		{
			if (*this->historical)
			{
				this->btnHisto->setCaption(this->btnHisto, this->strOff);
			}
			else
			{
				this->btnHisto->setCaption(this->btnHisto, this->strOn);
			}
			
			*this->historical = !*this->historical;
		}
		else if (this->cr->isOver(this->cr, this->btnExit))
		{
			choice = SystemMenu_CHOICE_QUIT;
		}
		else if (this->cr->isOver(this->cr, this->btnRestart))
		{
			choice = SystemMenu_CHOICE_RESTART;
		}
		else if (this->cr->isOver(this->cr, this->btnContinue))
		{
			choice = SystemMenu_CHOICE_CONTINUE;
		}
	}

	this->setVisible(this, FALSE);

	return choice;
}
