/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TBevel - Type Bevel

Hirarchie:
TObject -> TComponent -> TBevel
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TBevel reprsente une dnivellation dcorative.
 @since Release 1.0
*/

// Lignes normales
#define Bevel_NORMAL A_NORMAL

// Lignes inverses (effacer)
#define Bevel_REVERSE A_REVERSE

//Lignes XOR
#define Bevel_XOR A_XOR

//Lignes 2 fois plus paisses
#define Bevel_THICK1 A_THICK1

//Lignes  motif vertical
#define Bevel_SHADE_V A_SHADE_V

//Lignes  motif horizontal
#define Bevel_SHADE_H A_SHADE_H 				

//Lignes  motif de pente ngative
#define Bevel_SHADE_NS A_SHADE_NS

//Lignes  motif de pente positive
#define Bevel_SHADE_PS A_SHADE_PS

//Structure de l'objet de Type Bevel extension de TComponent
typedef struct Bevel {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Component*	(*clone)		(struct Component*);
	BOOL							(*equals)		(struct Component*, struct Component*);
	void							(*finalize)	(struct Bevel*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Bevel*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	int texture;	//La textures des lignes
	BOOL opact;		//L'opacit.
	void							(*setOpact)		(struct Bevel*, BOOL);
} TBevel;

//Dclaration des constructeurs
TBevel* TBevel0();

//Dclaration des mthodes
void				TBevel_finalize		(TBevel*);
void				TBevel_print			(TBevel*);
void				TBevel_setOpact		(TBevel*, BOOL);

/**
 * Ce constructeur instancie un objet de Type Component.
 @since Release 1.0
*/
TBevel* TBevel0 () {
	TBevel *this = (TBevel*)malloc(sizeof(TBevel));
	
	this->class  = "TBevel";
	this->super  = "TComponent";
	
	this->height 	= 0;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->texture	= Bevel_NORMAL;
	this->opact		= FALSE;
	
	this->clone 	 = TComponent_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TBevel_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TBevel_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	
	this->setOpact	 = TBevel_setOpact;

	return this;
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TBevel_print(TBevel* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	
	if (this->opact) {
		for (i=y0; i<y1; i++) {
			DrawLine(x0, i, x1, i, A_REVERSE);
		}
	}
	
	DrawLine(x0, y0, x1, y0, this->texture);
	DrawLine(x0, y1, x1, y1, this->texture);
	DrawLine(x0, (y0+1), x0, (y1-1), this->texture);
	DrawLine(x1, (y0+1), x1, (y1-1), this->texture);
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TBevel_finalize (TBevel* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode change l'opacit
 @param opact L'opacit
 @since Release 1.0
*/
void TBevel_setOpact(TBevel* this, BOOL opact) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->opact = opact;
		
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}
