/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TCanvas - Type Canvas

Hirarchie:
TObject -> TComponent -> TCanvas
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TCanvas permet de capturer et de dplacer des pixels.
 @since Release 1.0
*/

//Structure de l'objet de Type Canvas extension de TComponent
typedef struct Canvas {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Canvas*		(*clone)		(struct Canvas*);
	BOOL							(*equals)		(struct Canvas*, struct Canvas*);
	void							(*finalize)	(struct Canvas*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Canvas*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Canvas*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	BITMAP *content; //Le contenu du canvas
	void							(*saveContent)	(struct Canvas*);
	void							(*eraseContent)	(struct Canvas*);
} TCanvas;

//Dclaration des constructeurs
TCanvas* TCanvas0();

//Dclaration des mthodes
TCanvas*		TCanvas_clone					(TCanvas*);
BOOL				TCanvas_equals				(TCanvas*, TCanvas*);
void				TCanvas_finalize			(TCanvas*);
void				TCanvas_print					(TCanvas*);
void				TCanvas_setVisible		(TCanvas*, BOOL);
void				TCanvas_saveContent		(TCanvas*);
void				TCanvas_eraseContent	(TCanvas*);

/**
 * Ce constructeur instancie un objet de Type Canvas.
 @since Release 1.0
*/
TCanvas* TCanvas0 () {
	TCanvas *this = (TCanvas*)malloc(sizeof(TCanvas));
	
	this->class  = "TCanvas";
	this->super  = "TComponent";
	
	this->height 	= 0;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->content	= (BITMAP*)malloc(sizeof(BITMAP));
	
	this->clone 	 = TCanvas_clone;
	this->equals 	 = TCanvas_equals;
	this->finalize = TCanvas_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TCanvas_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TCanvas_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->saveContent  = TCanvas_saveContent;
	this->eraseContent = TCanvas_eraseContent;

	return this;
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TCanvas_print(TCanvas* this) {
	
	if (this->content->NumCols > 0 && this->content->NumRows > 0) {
		BitmapPut (this->x, this->y, this->content, ScrRect, A_REPLACE);
	}
}

/**
 * Cette mthode retourne un objet de Type Canvas qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TCanvas* TCanvas_clone(TCanvas* this) {

	TCanvas *clone;

	SCR_RECT rec;
	rec.xy.x0 = this->x;
	rec.xy.y0 = this->y;
	rec.xy.x1 = this->width - 1 + this->x;
	rec.xy.y1 = this->height - 1 + this->y;

	int i;
	
	clone = TCanvas0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	
	free(clone->content);
	
	clone->content	= (BITMAP*)malloc(BitmapSize(&rec));

	//memcpy(clone->content, this->content, BitmapSize(&rec));
	
	for (i = 0; i < BitmapSize(&rec); i++)
	{
		((char*)clone->content)[i] = ((char*)this->content)[i];
	}
	
	return clone;
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Dans le cas d'un objet de Type Canvas, une comparaison est tablie
 * entre les contenus des canvas.
 @param canvas0 L'objet  comparer.
 @return result - Vrai si les deux objets sont gaux.
 @since Release 1.0
*/
BOOL TCanvas_equals (TCanvas* this, TCanvas* canvas0) {
	
	BOOL result = FALSE;
	int i;

	this->saveContent(this);
	canvas0->saveContent(canvas0);

	if ( this->height == canvas0->height &&
			 this->width == canvas0->width
		 ) {
		result = TRUE;
		for (i=0; i<((this->height * this->width) / 8); i++) {
			if (this->content->Data[i] != canvas0->content->Data[i]) {
				result = FALSE;
			}
		}
	}
	
	return result;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TCanvas_finalize (TCanvas* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		free(this->img);
		free(this->content);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode sauvegarde le contenu du canvas.
 @since Release 1.0
*/
void TCanvas_saveContent (TCanvas* this) {
	
	SCR_RECT rec;
	
	if (this->visible) {	
		rec.xy.x0 = this->x;
		rec.xy.y0 = this->y;
		rec.xy.x1 = this->width - 1 + this->x;
		rec.xy.y1 = this->height - 1 + this->y;
		free(this->content);
		this->content = (BITMAP*)malloc(BitmapSize(&rec));
		BitmapGet (&rec, this->content);
	}
}

/**
 * Cette mthode efface le contenu du canvas.
 @since Release 1.0
*/
void TCanvas_eraseContent (TCanvas* this) {
	
	BOOL visible = this->visible;
	
	if (visible) {	
		TComponent_setVisible(this, FALSE);
	}
	
	free(this->content);
	this->content	= (BITMAP*)malloc(sizeof(BITMAP));
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode change la visibilit du composant.
 @param visible La visibilit du composant
 @since Release 1.0
*/
void TCanvas_setVisible(TCanvas* this, BOOL visible) {
	
	if (this->visible) {
		this->saveContent(this);
	}
	
	TComponent_setVisible(this, visible);
	
	if (visible) {
		this->saveContent(this);
	}
}
