/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TCheckButton - Type CheckButton

Hirarchie:
TObject -> TComponent -> TCaptionedComponent -> TButton -> TCheckButton
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TCheckButton reprsente un bouton que l'utilisateur peut cocher.
 @since Release 1.0
*/

//Structure de l'objet de Type CheckButton extension de TButton
typedef struct CheckButton {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct CheckButton*	(*clone)		(struct CheckButton*);
	BOOL								(*equals)		(struct CaptionedComponent*, struct Object*);
	void								(*finalize)	(struct CheckButton*);
	TString* 						(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct CheckButton*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct CaptionedComponent*, TString*);
	BOOL checked;
	void							(*setChecked)	(struct CheckButton*, BOOL);
} TCheckButton;

//Dclaration des constructeurs
TCheckButton* TCheckButton0();
TCheckButton* TCheckButton1(TString*);
TCheckButton* TCheckButton2(char*);

//Dclaration des mthodes
TCheckButton*					TCheckButton_clone			(TCheckButton*);
void									TCheckButton_finalize		(TCheckButton*);
void									TCheckButton_print			(TCheckButton*);
void									TCheckButton_setChecked	(TCheckButton*, BOOL);

/**
 * Ce constructeur instancie un objet de Type CheckButton.
 @since Release 1.0
*/
TCheckButton* TCheckButton0 () {
	TCheckButton *this = (TCheckButton*)malloc(sizeof(TCheckButton));
	
	this->class  = "TCheckButton";
	this->super  = "TButton";
	
	this->height 	= 15;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	= s("");
	this->checked = FALSE;
	
	this->clone 	 = TCheckButton_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TCheckButton_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TCheckButton_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TCaptionedComponent_setCaption;

	this->setChecked = TCheckButton_setChecked;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type CheckButton en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TCheckButton* TCheckButton1 (TString* caption) {
	
	TCheckButton *this = TCheckButton0();
	
	this->setCaption(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type CheckButton en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TCheckButton* TCheckButton2 (char* caption) {
	
	TCheckButton *this;
	TString *string;
	
	string = s(caption);
	
	this = TCheckButton1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type CheckButton qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TCheckButton* TCheckButton_clone(TCheckButton* this) {

	TCheckButton *clone;
	
	clone = TCheckButton0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->checked	= this->checked;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TCheckButton_finalize (TCheckButton* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode permet de cocher le bouton.
 @param checked Vrai si le composant doit tre coch
 @since Release 1.0
*/
void TCheckButton_setChecked (TCheckButton* this, BOOL checked) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	
	if (this->visible) {
		if (this->checked != checked) {	
			for (i=y0 + 2; i<y1 - 1; i++) {
				DrawLine(x0 + 2, i, x1 - 2, i, A_XOR);
			}
		}
	}
	
	this->checked = checked;
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TCheckButton_print(TCheckButton* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	
	TButton_print(this);
	
	if (this->checked) {
		for (i=y0 + 2; i<y1 - 1; i++) {
			DrawLine(x0 + 2, i, x1 - 2, i, A_XOR);
		}
	}
}
