/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TObject - Type Object

Hirarchie:
TObject
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TObject est la racine de la hirarchie des classes.  Toutes les classes
 * ont TObject comme superclasse. Tous les objets implmentent les mthodes de cette
 * classe.
 @since Release 1.0
*/

//Structure de l'objet de Type Object
typedef struct Object {
	char *class; //Le nom de la classe
	char *super; //Le nom de la superclasse
	struct Object* 	(*clone) 		(struct Object*);
	BOOL 						(*equals) 	(struct Object*, struct Object*);
	void 						(*finalize) (struct Object*);
	TString* 				(*toString) (struct Object*);
} TObject;

//Dclaration des constructeurs
TObject* TObject0();

//Dclaration des mthodes
TObject* 	TObject_clone 		(TObject*);
BOOL 			TObject_equals 		(TObject*, TObject*);
void 			TObject_finalize	(TObject*);
TString* 	TObject_toString 	(TObject*);

/**
 * Ce constructeur instancie un objet de Type Object.
 @since Release 1.0
*/
TObject* TObject0 () {
	TObject *this = (TObject*)malloc(sizeof(TObject));
	
	this->class  = "TObject";
	this->super  = "void";
	
	this->clone    = TObject_clone;
	this->equals   = TObject_equals;
	this->finalize = TObject_finalize;
	this->toString = TObject_toString;
	
	return this;
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Pour la classe TObject, elle implmente la plus grande discrimination
 * possible entre objets, car pour qu'elle retourne vrai il faut que la valeur
 * passe en paramtre rfre directement  l'objet.
 @param object0 L'objet avec qui comparer.
 @return equal - Vrai si les objets sont gaux.
 @since Release 1.0
*/
BOOL TObject_equals (TObject* this, TObject* object0) {

	BOOL result = (this == object0);
	
	return result;
}

/**
 * Cette mthode retourne une chane de caractres reprsentant l'objet.
 * Pour la classe TObject, il s'agit du nom de sa classe et de sa superclasse;
 @return string - La chane de caractre reprsentant l'objet.
 @since Release 1.0
*/
TString* TObject_toString (TObject* this) {

	TString *string;
	
	string = TString4(3, this->class, "@", this->super);
	
	return string;
}

/**
 * Cette mthode retourne un objet de Type Object qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TObject* TObject_clone (TObject* this) {

	TObject *clone;
	
	clone = TObject0();
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TObject_finalize(TObject* this) {
		
		TRY {
			free(this);	
		}
		ONERR {}
		ENDTRY
}
