/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TTextField - Type TextField

Hirarchie:
TObject -> TComponent -> TCaptionedComponent -> TTextField
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TTextField reprsente un champ texte.
 @since Release 1.0
*/

//Structure de l'objet de Type TextField extension de TCaptionedComponent
typedef struct TextField {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct TextField*	(*clone)		(struct TextField*);
	BOOL							(*equals)		(struct CaptionedComponent*, struct Object*);
	void							(*finalize)	(struct TextField*);
	TString* 					(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct TextField*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct CaptionedComponent*, TString*);
	BOOL masked;			//Indique si le texte est masqu
	char mask;				//Le masque
	void							(*setMasked)	(struct TextField*, BOOL);
	void							(*setMask)		(struct TextField*, char);
	void							(*use)				(struct TextField*);
} TTextField;

//Dclaration des constructeurs
TTextField* TTextField0();
TTextField* TTextField1(TString*);
TTextField* TTextField2(char*);

//Dclaration des mthodes
TTextField*						TTextField_clone			(TTextField*);
void									TTextField_finalize		(TTextField*);
void									TTextField_print			(TTextField*);
void									TTextField_setMasked	(TTextField*, BOOL);
void									TTextField_setMask		(TTextField*, char);
void									TTextField_use				(TTextField*);

/**
 * Ce constructeur instancie un objet de Type TextField.
 @since Release 1.0
*/
TTextField* TTextField0 () {
	TTextField *this = (TTextField*)malloc(sizeof(TTextField));
	
	this->class  = "TTextField";
	this->super  = "TCaptionedComponent";
	
	this->height 	= 15;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	= s("");
	this->masked	= FALSE;
	this->mask		= '*';
	
	this->clone 	 = TTextField_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TTextField_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TTextField_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TCaptionedComponent_setCaption;

	this->use				= TTextField_use;
	this->setMasked = TTextField_setMasked;
	this->setMask		= TTextField_setMask;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type TextField en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TTextField* TTextField1 (TString* caption) {
	
	TTextField *this = TTextField0();
	
	this->setCaption(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type TextField en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TTextField* TTextField2 (char* caption) {
	
	TTextField *this;
	TString *string;
	
	string = s(caption);
	
	this = TTextField1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type TextField qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TTextField* TTextField_clone(TTextField* this) {

	TTextField *clone;
	
	clone = TTextField0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->caption	= c(this->caption);
	clone->masked		= this->masked;
	clone->mask			= this->mask;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TTextField_finalize (TTextField* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TTextField_print(TTextField* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	TString *text;
	
	for (i=y0; i<y1; i++) {
		DrawLine(x0, i, x1, i, A_REVERSE);
	}
	
	DrawLine(x0, y0, x1, y0, A_NORMAL);
	DrawLine(x0, y1, x1, y1, A_NORMAL);
	DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
	DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	
	text = this->caption->clone(this->caption);
	
	if (this->masked) {
		for (i = 0; i < text->length(text); i++) {
			text->string[i] = this->mask;
		}
	}
	
	DrawStr((x0+2), (y0+2), text->string, A_NORMAL);
	
	text->finalize(text);
}

/**
 * Cette mthode permet  l'utilisateur d'utiliser le composant.
 @since Release 1.0
*/
void TTextField_use(TTextField* this) {
	
	int key;
	TString *caption = s("");
	char *array;
	int maxLength;
	int ptr;
	int cptW = CaptionedComponent_getFontWidth();
	
	maxLength = (this->width - 4) / cptW;
	
	array = (char*)malloc(maxLength + 1);
	
	strcpy(array, this->caption->string);
	
	ptr = this->caption->length(this->caption);
	
	do {
		key = ngetchx();
		
		if (key >= ' ' && key <= '~' && ptr < maxLength) {
			array[ptr] = key;
			ptr++;
			array[ptr] = '\0';
		}
		else if (key == KEY_BACKSPACE) {
			if (ptr > 0) {
				array[ptr-1] = '\0';
				ptr--;
			}
		}
		else if (key == KEY_CLEAR) {
			array[0] = '\0';
			ptr = 0;
		}
		
		caption->finalize(caption);
		caption = s(array);
		
		this->setCaption(this, caption);
	}	while (key != KEY_ENTER);
	
	free(array);
}

/**
 * Cette mthode permet de spcifier s'il y a prsence de masque ou non.
 @param masked Spcifie s'il y a prsence de masque
 @since Release 1.0
*/
void TTextField_setMasked(TTextField* this, BOOL masked) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->masked = masked;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode permet de spcifier le masque.
 @param mask Le masque
 @since Release 1.0
*/
void TTextField_setMask(TTextField* this, char mask) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->mask = mask;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}
